<?php

// functions used for different ways of searching and displaying search results

// search entities with name plus subname exactly as search string
function search_exact_entities($string)
{
	global $con;
	
	// substituting 'non-standard' characters with their 'standard' equivalents and ignoring case
	// name plus sub name of entity must be 'identical' to search string
	$query_string = "
	SELECT `entities`.`name` AS `found_string`, `entities`.*
	FROM `entities`
	WHERE `entities`.`visibility` = 1 AND CAST(`entities`.`name` AS CHAR CHARACTER SET utf8) COLLATE utf8_unicode_ci LIKE ?
	ORDER BY `entities`.`name` ASC, `entities`.`sub_name` ASC";
	
	// writing search results into output array
	$entities_data = array();
	
	$query = $con->prepare($query_string);
	$query->bind_param('s', $string);
	$query->execute();
	
	bind_array($query, $row);
	while ($query->fetch())
		$entities_data[$row["ID"]] = array("ID" => $row["ID"], "name" => $row["name"], "sub_name" => $row["sub_name"], "geography" => $row["geography"], "hierarchy" => $row["hierarchy"]);

	return $entities_data;
}

// search entities with name plus subname containing exact search string; if $from_start == TRUE, entity name must start with search string
function search_entities($string, $from_start = FALSE)
{
	global $con, $T;
	
	$string = ($from_start?"":"%").$string."%";
	
	// substituting 'non-standard' characters with their 'standard' equivalents and ignoring case
	// name plus sub name of entity must contain whole search string
	$query_string = "
	SELECT CONCAT(`entities`.`name`, IF(ISNULL(`entities`.`sub_name`), '', CONCAT(' ', `entities`.`sub_name`))) AS `found_string`, `entities`.*
	FROM `entities`
	WHERE `entities`.`visibility` = 1 AND CAST(CONCAT(`entities`.`name`, ' ', IFNULL(`entities`.`sub_name`, '')) AS CHAR CHARACTER SET utf8) COLLATE utf8_unicode_ci LIKE ?
	ORDER BY `entities`.`name` ASC, `entities`.`sub_name` ASC";
	
	// writing search results into output array
	$entities_data = array();
	$found_strings = array();
	
	$query = $con->prepare($query_string);
	$query->bind_param('s', $string);
	$query->execute();

	bind_array($query, $row);
	while ($query->fetch())
	{
		$entities_data[$row["ID"]] = array("ID" => $row["ID"], "name" => $row["name"], "sub_name" => $row["sub_name"], "geography" => $row["geography"], "hierarchy" => $row["hierarchy"]);
	}

	// adding search method "found in name" for all found entities
	if (count($entities_data) > 0)
		$found_strings = array_fill(0, count($entities_data), $T["found_in_name"]);
	
	return array($entities_data, $found_strings);
}

// search entity properties containing search string; return found entities
function search_entity_properties($string)
{
	global $con, $T;
	
	$string = "%".$string."%";
	
	// substituting 'non-standard' characters with their 'standard' equivalents and ignoring case
	// name plus sub name of entity must contain whole search string
	$entities_query_string = "
	SELECT DISTINCT `entity_properties`.`entity_ID`, `entity_properties`.`value` AS `found_string`
	FROM `entity_properties` 
	LEFT JOIN `entities` ON `entity_properties`.`entity_ID` = `entities`.`ID`
	WHERE `entities`.`visibility` = 1 AND CAST(`entity_properties`.`value` AS CHAR CHARACTER SET utf8) COLLATE utf8_unicode_ci LIKE ?";
	
	// writing search results into output array
	$found_strings = array();
	$entities = array();

	$query = $con->prepare($entities_query_string);
	$query->bind_param('s', $string);
	$query->execute();

	bind_array($query, $row);
	while ($query->fetch())
		$entities[] = $row["entity_ID"];
		
	// adding search method "found in entity basic properties" for all found entities
	$entities_data = get_entities_specs($entities, TRUE);
	if (count($entities_data) > 0)
		$found_strings = array_fill(0, count($entities_data), $T["found_in_basic_properties"]);
		
	return array($entities_data, $found_strings);
}

// search elements of geography structure containing search string; return entities belonging to found geography structure elements
function search_geo($string)
{
	global $con, $T;
	
	$string = "%".$string."%";
	
	// substituting 'non-standard' characters with their 'standard' equivalents and ignoring case
	// name plus sub name of geographical structure element must contain whole search string
	$query_string = "
	SELECT `ID`, `name` AS `found_string`
	FROM `geographical_structure`
	WHERE CAST(`name` AS CHAR CHARACTER SET utf8) COLLATE utf8_unicode_ci LIKE ?";

	// get ID of geographical element with zero level ("master" geographical element)
	$all_hies = reset(get_hies('0'));
	$all_hies = $all_hies["ID"];
	
	// finding entities for geographical elements found and writing found entities into output array
	$entities_data = array();
	$found_strings = array();

	$query = $con->prepare($query_string);
	$query->bind_param('s', $string);
	$query->execute();
	
	// for each geographical element found
	bind_array($query, $row);
	while ($query->fetch())
	{
		$geo = $row["ID"];
		// find entities
		$entities = get_entities($geo, $all_hies);
		// append found entities at the end of array (replacing duplicates)
		$entities_data = $entities_data + $entities;
	}
	
	// adding search method "found in geographical structure element name" for all found entities
	if (count($entities_data) > 0)
		$found_strings = array_fill(0, count($entities_data), $T["found_in_geo_levels"]);
		
	return array($entities_data, $found_strings);	
}

// search elements of hierarchy structure containing search string; return entities belonging to found hierarchy structure elements
function search_hie($string)
{
	global $con, $T;
	
	$string = "%".$string."%";

	// substituting 'non-standard' characters with their 'standard' equivalents and ignoring case
	// name plus sub name of hierarchical structure element must contain whole search string
	$query_string = "
	SELECT `ID`, `name` AS `found_string`
	FROM `hierarchical_structure`
	WHERE CAST(`name` AS CHAR CHARACTER SET utf8) COLLATE utf8_unicode_ci LIKE ?";
	
	// get ID of hierarchical element with zero level ("master" hierarchical element)
	$all_geos = reset(get_geos('0'));
	$all_geos = $all_geos["ID"];
	
	// finding entities for hierarchical elements found and writing found entities into output array
	$entities_data = array();
	$found_strings = array();

	$query = $con->prepare($query_string);
	$query->bind_param('s', $string);
	$query->execute();
	
	// for each hierarchical element found
	bind_array($query, $row);
	while ($query->fetch())
	{
		$hie = $row["ID"];
		// find entities
		$entities = get_entities($all_geos, $hie);
		// append found entities at the end of array (replacing duplicates)
		$entities_data = $entities_data + $entities;
	}
		
	// adding search method "found in hierarchical structure element name" for all found entities
	if (count($entities_data) > 0)
		$found_strings = array_fill(0, count($entities_data), $T["found_in_hie_levels"]);
		
	return array($entities_data, $found_strings);	
}

// display search results
function draw_search_results($final_elements, $header_text, $final_search_strings = FALSE)
{
	global $T, $GS, $geo_table, $hie_table, $data_specs, $indicators_specs, $ratings_specs;
	
	// if one column is defined as 'main rating', this rating for each entity will be displayed in search results
	if ($GS["main_rating"])
	{
		$elements_string_array = array();
		foreach ($final_elements as $ID => $data)
			$elements_string_array[] = $ID;
		$elements_string = implode(",", $elements_string_array);
		
		// get rating for entities found
		$ratings = get_ranking_values($GS["main_rating"], $elements_string, $GS["newest_year"], 'order', TRUE);
		
		$ID = substr($GS["main_rating"], 1);
		// get specification of main rating
		$specs = $ratings_specs[$ID];
	}
	
	echo '<div class="body-content-wrapper">';
	echo '<div class="body-content">';
	echo '<div class="element-wrapper">';
	if ($header_text)
		echo '<h2>'.$header_text.'</h2>';
	echo '<table id="search-table">';

	$c = 0; // search methods
	$d = 1; // elements found
	$last_text = '';
	foreach ($final_elements as $ID => $data)
	{
		if ($final_search_strings) // search method was specified
			if ($final_search_strings[$c] !== $last_text) // previous entit was found by different search method than current
			{
				// displaying header informing that following search results were found by different method then previous
				if ($GS["main_rating"]) $colspan = 4; else $colspan = 3;
				echo '<tr><td colspan="'.$colspan.'"><h4>'.$final_search_strings[$c].'</h4></td></tr>';
				$last_text = $final_search_strings[$c];
				$d = 1;
			}
			
		echo '<tr class="'.($d % 2 == 1?'odd':'even').'">'; // background color of row
		echo '<td class="st-number">'.($c+1).'</td>'; // order of found entities
		echo '<td class="st-name"><a href="/profile/'.$ID.'/'.$GS["newest_year"].'">'.$data["name"].($data["sub_name"]?'<span>'.$data["sub_name"].'</span>':'').'</a></td>'; // entity name, click leading to its profile
		if ($GS["main_rating"])
		{
			$rating = $ratings[1][$ID][$GS["main_rating"]]; // fetch rating from array
			$bg_color = get_rating_color($rating, $specs["minimum"], $specs["maximum"]); // format rating background
			$rating_string = format_rating($rating, $specs["datatype"], $specs["minimum"], $specs["maximum"]); // format rating text
			echo '<td class="st-rating">';
			if ($rating !== NULL)
				echo '<span style="color:'.$GS["rating_text_color"].'; background-color:'.$bg_color.'">'.$rating_string.'</span></td>'; // display formatted rating
			else
				echo $GS["missing_sign"];
		}
		echo '<td class="st-details">'.$hie_table[$data["hierarchy"]]["name"].', '.$geo_table[$data["geography"]]["name"].'</td>'; // details of entity - geographical and hierarchical structure
		echo '</tr>';
		$c++;
		$d++;
	}
	
	echo '</table>';
	echo '</div>';
	echo '</div>';
	echo '</div>';
}

?>